# Global settings for CLIbrary.

class style:
	setting_darkMode = False # Enables global dark mode.
	setting_plainMode = False # Disables styling.
	setting_caseSensitive = False # Case sensitive strings.

class data:
	setting_fileExtension = ".pickle" # Defines a file extension for aDump and aLoad. Default value based on Python's module pickle.